'-- LM75-Ser.bas ------------------------------------------------ >>fst'12<< -
' - liest den 9-Bit Temperatursensor LM75 ueber die serielle Schnittstelle aus
' - Spannungsversorgung des LM75 aus der Schnittstelle (Spannungsregler)
' - Quelltext fr FreeBasic / Windows ab Win95
' - Compiler = FreeVBasic kompilieren mit Parameter -s console
' - Mit i2c-BUS Befehlssatz in separaten SUBs / FUNCTIONs
' - Verbindungen:
'   -- LM75 --                --- COM ---  -- Zweck ------------------
'   Pin1 (SDA) --o-[10kOhm]-< Pin 7 (RTS)  = Datenleitung vom PC
'                +----------> Pin 8 (CTS)  = Datenleitung zum PC
'   Pin2 (SCL) --[100kOhm]--< Pin 4 (DTR)  = Taktleitung
'   Pin3 (O.S.)        nicht verbunden     = Schaltausgang LM75
'   Pin4 (GND) -----------o-- Pin 5 (GND)  = Masse
'   Pin5 (A2)             +-  Pin 5 (GND)  = Slave-Adress Einsteller 2
'   Pin6 (A1)             +-  Pin 5 (GND)  = Slave-Adress Einsteller 1
'   Pin7 (A0)             +-  Pin 5 (GND)  = Slave-Adress Einsteller 0
'   Pin8 (+Vs) -->|--UReg5V-- Pin 3 (TxD)  = Spannungsversorgung
'-- V 20120510 --------------------------------- http://www.FrankSteinberg.de -

#INCLUDE ONCE "windows.bi"
Declare SUB AbfragePortNr (As String)
Declare SUB ComOpen (ByVal As String)
Declare FUNCTION Celsius (ByVal As UByte, ByVal As UByte) As Single
Declare Sub Anzeige(ByVal As String, ByVal As Single, ByVal as UByte, ByVal as UByte,_
                    ByVal As UInteger, ByVal As UInteger, ByVal As UInteger)
Declare SUB LM75reset ()
'i2c Protokoll:
Declare SUB i2cStart ()
Declare SUB i2cStop ()
Declare SUB i2cAckMaster ()
Declare SUB i2cNoAckMaster ()
Declare SUB i2cWriteByte (ByVal As UByte)
Declare FUNCTION i2cReadByte As UByte
Declare FUNCTION i2cAckSlave As Byte
'i2c Low-Level:
Declare SUB SVC (ByVal As Byte)
Declare SUB SCL (ByVal As Byte)
Declare SUB SDA (ByVal As Byte)
Declare FUNCTION SDAin As Byte

Dim Shared hCom As Handle       'Handle des Comports
Dim PortNr As String            'Nummer des Comports (COM1 - COM255 mglich)
Dim As UByte Byte1, Byte2       'DatenBytes wie vom LM75 geliefert
Dim GradC As Single             'Messwert in Grad Celsius; Auflsung 0,5 Grad
Dim SDALocked As UInteger       'Fehlerzhler SDA Dauer-Low
Dim AckSlaveFehler As UInteger  'Fehlerzhler fr Acknowledge vom LM75
Dim FehlerAnzahl As UInteger    'Fehlerzhler Summe

'Portnummer entweder per Parameterbergabe oder per Abfrage:
PortNr = Command
AbfragePortNr(PortNr)

'Startpunkt fr Chip-Reset
Init:

'Serielle Schnittstelle ffnen:
ComOpen(ByVal PortNr)

'Versorgungsspannung ueber TxD (Pin 3) einschalten (optional):
SVC(1)      'Spannungsversorgung TxD->SVC (Pin8) high
Sleep(200)  '200 Millisekunden Zeit zum Stabilisieren der Spannung

'Stopsequenz ausgeben:
i2cStop()


'*** Hauptschleife
DO  

  '90 Millisekunden Wartezeit fuer LM75:
 Sleep(90)

 'Startsequenz ausgeben:
 i2cStart()

 'Adressbyte des LM75 ausgeben:
 i2cWriteByte(145)
 '145 = 1 0 0 1 0 0 0 1 binaer
 '      ^ ^ ^ ^         : feste Adresse des LM75
 '              ^ ^ ^   : variable Adresse des LM75, wie
 '                        Pin7/Pin6/Pin5 = A0/A1/A2 verbunden
 '                    ^ : 0=Master (PC) schreibt anschliessend
 '                        1=Master (PC) liest anschliessend

 'Acknowledge vom LM75 lesen und Anzahl der Fehler speichern:
 IF i2cAckSlave THEN AckSlaveFehler = AckSlaveFehler + 1

 'Datenbyte 1 lesen:
 Byte1 = i2cReadByte

 'Acknowledge vom Master ausgeben:
 i2cAckMaster

 'Datenbyte 2 lesen:
 Byte2 = i2cReadByte

 'No Acknowledge vom Master:
 i2cNoAckMaster
 'Wenn der LM75 SDA low hlt, ist der Chip gelockt (Datenblatt Seite 8, letzter Absatz):
 ' Die dort beschrieben Entstrung funktioniert leider nur selten, deshalb wird 
 ' in der SUB "LM75reset" den Spannung komplett an/abgeschaltet. 
 If SDAin = 0 Then SDALocked = SDALocked + 1
 
 'Stopsequenz ausgeben:
 i2cStop
  
 'Bytes in C umrechnen:
 GradC = Celsius(Byte1, Byte2)

 'Bildschirmanzeige: 
 Anzeige(PortNr, GradC, Byte1, Byte2, AckSlaveFehler, SDALocked, FehlerAnzahl)
 
 'LM75 bei Fehler resetten: 
 If (SDALocked + AckSlaveFehler) > FehlerAnzahl Then  'wenn neuer Fehler aufgetreten ...
   FehlerAnzahl = SDALocked + AckSlaveFehler          'Fehleranzahl aktualisieren
   LM75reset                                          'Spannung aus, ComPort schlieen
   If InKey <> "" Then End                            'Ende, wenn Taste gedrckt
   GoTo Init                                          'zum Programmanfang springen
 EndIf
 
Loop WHILE INKEY = ""  '*** Hauptschleife wiederholen bis Tastendruck

'Versorgungsspannung ausschalten:
SVC(0)

'Serielle Schnittstelle schlieen:
closeHandle(hCom)

End  '*** Ende des Programms


Sub AbfragePortNr(PortNr As String)
'Portnummer abfragen, wenn nicht als Parameter eingegeben:
 
 Cls
 If PortNr = "" Then
   Locate 12,4
   Line Input "Portnummer eingben: LM75 ist angeschlossen an COM", PortNr
   Cls
   PortNr = "Com" + PortNr
 EndIf
	
End Sub


Sub ComOpen(ByVal PortNr As String)
	'Serielle Schnittstelle ffnen:

	hCom = CreateFile("\\.\"+PortNr, GENERIC_READ Or GENERIC_WRITE, 0, NULL, OPEN_EXISTING, 0, NULL)
	'Fehlerbehandlung, falls ffnen fehlschlgt:
	If hCom = INVALID_HANDLE_VALUE Then
	  MessageBox 0,"Fehler beim ffnen von  " & PortNr," LM75_FB   >>fst'12<<", MB_ICONERROR
	  End
	End If
	'Steuerleitungen low:
	EscapeCommFunction(hCom, CLRRTS)    'RTS -12V
	EscapeCommFunction(hCom, CLRDTR)    'DTR -12V
End Sub


FUNCTION Celsius (ByVal Byte1 as UByte, ByVal Byte2 as UByte) as Single
 'Datenbytes vom LM75 nach C umrechnen
 
 Dim x as Single = Byte1
 
 IF (Byte1 AND 128) THEN x = x - 256  'Vorzeichen-Bit (Byte1,Bit7) auswerten
 If (Byte2 AND 128) THEN x = x + .5   'Halbgrad-Bit   (Byte2,Bit7) auswerten
 Return x

END FUNCTION


FUNCTION MessProSek as Uinteger
 'Messungen pro Sek. ermitteln:
 
 Static z As UInteger
 Static zSumme As UInteger
 Static T1 As String 
 z = z + 1
 IF T1 <> TIME THEN 
 	 zSumme = z
 	 z = 0
 	 T1 = TIME
 EndIf
 Return zSumme
 
END Function


Sub Anzeige(ByVal PortNr As String, ByVal GradC As Single, ByVal Byte1 as UByte,_
	           ByVal Byte2 as UByte, ByVal AckSlaveFehler As UInteger,_
	           ByVal SDALocked As UInteger, ByVal FehlerAnzahl As UInteger)
	'Bildschirmausgabe:
	
	Locate 1,1
	Print
	Print " Temperaturmessung          >>fst'12<<"
	Print
	Print "  LM75  <->  i2c-BUS  <->  " + PortNr
 LOCATE 9, 1
 COLOR 15, 0
 Print USING " Temperatur           = ####.# Grad C"; GradC
 COLOR 7, 0
 Print : Print : Print
 Print USING " Daten-Byte 1         = ####"; Byte1
 Print USING " Daten-Byte 2         = ####"; Byte2
 Print
 Print USING " Messungen/Sek        = ####"; MessProSek
 Print
 Print USING " Acknowledge vom LM75 =##### Fehler"; AckSlaveFehler
 Print USING " LM75 SDA-Lock        =##### Fehler"; SDALocked
 Print USING " Anzahl Chip-Resets   =#####"; FehlerAnzahl 
	
End Sub


SUB i2cAckMaster
'"Acknowledge" auf den i2c-BUS geben

 SDA(0)  'SDA low  ausgeben
 SCL(1)  'SCL high ausgeben
 SCL(0)  'SCL low  ausgeben

END SUB


FUNCTION i2cAckSlave as Byte
'"Acknowledge" des Slave vom i2c-BUS lesen

 SDA(0)               'SDA low ausgeben
 SCL(1)               'SCL high ausgeben
 i2cAckSlave = SDAin  'Acknowledge-Bit vom Slave lesen (FUNCTION SDAin)
 'SDA(0)               'SDA wieder in den Ausgangszustand
 SCL(0)               'SCL low  ausgeben

END FUNCTION


FUNCTION i2cReadByte as UByte
 'ein Byte vom i2c-BUS lesen
 
 Dim as UByte BitW, ByteW, i
 BitW = 128                           'erstes Bit hat den Wert 128 (MSB)
 FOR i = 0 TO 7                       '8 Bits schreiben:
  SCL(1)                              'SCL high ausgeben
  IF SDAin THEN ByteW = ByteW + BitW  'ein Bit von SDA lesen (FUNCTION SDAin)
  BitW = BitW \ 2                     'Bit-Wert fuer naechste Abfrage aendern
  SCL(0)                              'SCL low ausgeben
 NEXT
 i2cReadByte = ByteW                  'Ergebnis ausgeben

END FUNCTION


SUB i2cNoAckMaster
'"No Acknowledge" auf den i2c-BUS geben

 SDA(1)  'SDA high ausgeben
 SCL(1)  'SCL high ausgeben
 SCL(0)  'SCL low  ausgeben

END SUB


SUB i2cWriteByte (ByVal ByteW as UByte)
'ein Byte auf den i2c-BUS schreiben
 
 Dim as UByte BitW, i
 BitW = 128                 'erstes Bit hat den Wert 128 (MSB)
 
 FOR i = 0 TO 7             '8 Bits schreiben:
  IF (ByteW AND BitW) THEN  'wenn Bit=1 ...
    SDA(1)                  'dann SDA high ausgeben
    'If SDAin = 0 Then LM75reset
   ELSE                     'wenn Bit=0 ...
    SDA(0)                  'dann SDA low  ausgeben
  END IF
  BitW = BitW \ 2           'Bit-Wert fuer naechste Abfrage aendern
  SCL(1)                    'SCL high ausgeben
  SCL(0)                    'SCL low  ausgeben
 NEXT i

END SUB


SUB i2cStart
 'Startsequenz auf den i2c-BUS schreiben

 SDA(0)  'SDA low  ausgeben
 SCL(0)  'SCL low  ausgeben

END SUB


SUB i2cStop
 'Stopsequenz auf den i2c-BUS schreiben

 SCL(1)  'SCL high ausgeben
 SDA(1)  'SDA high ausgeben

END SUB


SUB SCL (ByVal BitW as Byte)
'gibt "high" oder "low " auf die Taktleitung (SCL) des i2c-BUS

 IF BitW THEN                        'wenn BitW 1 uebergeben ...
   EscapeCommFunction(hCom, SETDTR)  'DTR -> SCL high
 ELSE                                'wenn BitW 0 uebergeben ...
   EscapeCommFunction(hCom, CLRDTR)  'DTR -> SCL low
 END IF
 Sleep(1)                            '1 Millisekunde warten

END SUB


SUB SDA (ByVal BitW as Byte)
'gibt "high" oder "low " auf die Datenleitung (SDA) des i2c-BUS

 IF BitW THEN                        'wenn BitW 1 uebergeben ...
   EscapeCommFunction(hCom, SETRTS)  'RTS -> SDA high
 ELSE                                'wenn BitW 0 uebergeben ...
   EscapeCommFunction(hCom, CLRRTS)  'RTS -> SDA low
 END IF
 'Sleep(1)                           '1 Millisekunde warten

END SUB


FUNCTION SDAin as Byte
 'liest ein Bit vom i2c-BUS

 Dim modemStat As dword                'Rckgabewert der Eingangsleitungen

 SDA(1)                                'der mit SDA verbundene Ausgangmuss high sein, damit ueber
                                       'den Eingang gelesen werden kann (LM75 zieht SDA auf Masse).
 Sleep(2)                              '2 Millisekunden warten
 GetCommModemStatus(hCom, @modemStat)  'Eingnge vom Comport lesen
 'Zustand von CTS (=DATA OUT) lesen und auswerten:
 '1 = high erkannt (SDA -> CTS)
 IF (modemStat And MS_CTS_ON) THEN SDAin = 1  

END FUNCTION


SUB SVC (ByVal Flag as Byte)
'gibt "high" oder "low " auf TxD (Pin 3) des COMPorts
'zur Spannungsversorgung des angeschlossenen Bauteils (hier LM75)

 IF Flag THEN
   EscapeCommFunction(hCom, SETBREAK)  'TxD -> [Spannungsregler] -> +Vs high
  ELSE
   EscapeCommFunction(hCom, CLRBREAK)  'TxD -> [Spannungsregler] -> +Vs low
 END IF

END Sub


Sub LM75reset
 
 SVC(0)                         'Versorgungsspannung ausschalten
 closeHandle(hCom)              'Serielle Schnittstelle schlieen
 Sleep(100)                     '0,1 Sek. warten
 
End Sub
